"""
MIT License

Copyright (c) 2023 dbrennand

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import requests
import os
from typing import Union
from json.decoder import JSONDecodeError


class VirustotalError(Exception):
    """Class for VirusTotal API errors."""

    def __init__(self, response: requests.Response) -> None:
        """Initialisation for `VirustotalError`.

        Args:
            response (requests.Response): From a failed API request to the VirusTotal API.
        """
        self.response = response

    def __str__(self) -> str:
        """String dunder method for `VirustotalError`.

        Returns:
            str: Containing the error code, HTTP status code and error message from a failed VirusTotal API request.
        """
        error = self.error()
        return f"Error {error.get('code', 'Unknown')} ({self.response.status_code}): {error.get('message', 'No message')}"

    def error(self) -> dict:
        """Retrieve the error that occurred from a failed VirusTotal API request.

        https://developers.virustotal.com/v2.0/reference/api-responses

        https://developers.virustotal.com/reference/errors

        Returns:
            dict: Containing the error code and message returned from the VirusTotal API (if any) otherwise, returns an empty dictionary.
        """
        # Attempt to decode JSON as the v3 VirusTotal API returns the error message as JSON
        # Fallback to an empty dict if error is somehow missing
        try:
            return self.response.json().get("error", dict())
        except ValueError:
            # Catch ValueError if JSON fails to be deserialized or there is no JSON
            # Most likely using the v2 VirusTotal API
            # Check there is response text, if not, return an empty dict
            if self.response.text:
                return dict(message=self.response.text)
            else:
                return dict()


class VirustotalResponse(object):
    """Response class for VirusTotal API requests."""

    def __init__(self, response: requests.Response):
        """Initialisation for `VirustotalResponse`.

        Args:
            response (requests.Response): From a successful API request to the VirusTotal API.
        """
        self.response = response

    @property
    def headers(self) -> dict:
        """Retrieve the HTTP headers of a VirusTotal API request.

        Returns:
            dict: The HTTP headers of the requests.Response object.
        """
        return self.response.headers

    @property
    def status_code(self) -> int:
        """Retrieve the HTTP status code of a VirusTotal API request.

        Returns:
            int: The HTTP status code of the requests.Response object.
        """
        return self.response.status_code

    @property
    def text(self) -> str:
        """Retrieve the HTTP text response of a VirusTotal API request.

        Returns:
            str: The HTTP text response of the requests.Response object.
        """
        return self.response.text

    @property
    def requests_response(self) -> requests.Response:
        """Retrieve the `requests.Response` object of a VirusTotal API request.
        You may want to access this property if you wanted to read other aspects of the response such as cookies.

        Returns:
            requests.Response: From a successful API request to the VirusTotal API.
        """
        return self.response

    @property
    def links(self) -> Union[dict, None]:
        """Retrieve the value of the key 'links' in the JSON response from a VirusTotal API request.

        https://developers.virustotal.com/reference/collections

        NOTE: Links are not retrieved for objects inside 'data'.

        Returns:
            Union[dict, None]: The links used to retrieve the next set of objects (if any), otherwise, returns `None`.
        """
        return self.json().get("links", None)

    @property
    def meta(self) -> Union[dict, None]:
        """Retrieve the value of the key 'meta' in the JSON response from a VirusTotal API request.

        https://developers.virustotal.com/reference/collections

        Returns:
            Union[dict, None]: The metadata about the object(s) (if any), otherwise, returns `None`.
        """
        return self.json().get("meta", None)

    @property
    def cursor(self) -> Union[str, None]:
        """Retrieve the value of the key 'cursor' in the JSON response value 'meta' from a VirusTotal API request.

        https://developers.virustotal.com/reference/collections

        Returns:
            Union[str, None]: The cursor used to retrieve related object(s), otherwise, returns `None`.
        """
        try:
            return self.meta.get("cursor", None)
        # Catch AttributeError that occurs when attempting to call attribute 'get' on None
        # which is raised if the 'meta' key is not present in the JSON response
        except AttributeError:
            return None

    @property
    def data(self) -> Union[dict, list, str, None]:
        """Retrieve the value of the key 'data' in the JSON response from a VirusTotal API request.

        https://developers.virustotal.com/reference/objects

        Returns:
            Union[dict, list, str, None]: The data from a VirusTotal API request (if any) otherwise, returns `None`.
        """
        return self.json().get("data", None)

    @property
    def object_type(self) -> Union[list, str, None]:
        """Retrieve the object type(s) in the JSON response from a VirusTotal API request.

        https://developers.virustotal.com/reference/objects

        https://developers.virustotal.com/reference/collections

        Returns:
            Union[list, str, None]: A list or string depending on the number of objects returned (if any) otherwise, returns `None`.
        """
        data = self.data
        # Check if data contains more than one object
        if isinstance(data, list):
            object_list = []
            for data_object in data:
                data_object_type = data_object.get("type", None)
                object_list.append(data_object_type)
            return object_list
        # Data contains only one object
        elif isinstance(data, dict):
            return data.get("type", None)
        else:
            return None

    @property
    def response_code(self) -> Union[int, None]:
        """Retrieve the value of the key 'response_code' in the JSON response from a VirusTotal v2 API request.

        https://developers.virustotal.com/v2.0/reference/api-responses

        Returns:
            Union[int, None]: The response_code from the VirusTotal API (if any), otherwise, returns `None`.
        """
        return self.json().get("response_code", None)

    def json(self, **kwargs) -> dict:
        """Retrieve the JSON response from a VirusTotal API request.

        Args:
            **kwargs: Identical to `json.loads(**kwargs)`.

        Returns:
            dict: JSON response from a VirusTotal API request.
        """
        try:
            return self.response.json(**kwargs)
        except JSONDecodeError:
            return dict()


class Virustotal(object):
    """Interact with the public VirusTotal v3 and v2 APIs.

    https://developers.virustotal.com/v3.0/reference

    https://developers.virustotal.com/v2.0/reference
    """

    def __init__(
        self,
        API_KEY: str = None,
        API_VERSION: Union[int, str] = 3,
        PROXIES: dict = None,
        TIMEOUT: float = None,
    ):
        """Interact with the public VirusTotal v3 and v2 APIs.

        Args:
            API_KEY (str, optional): The API key used to interact with the VirusTotal v3 and v2 APIs.
                Alternatively, the environment variable `VIRUSTOTAL_API_KEY` can be provided.
            API_VERSION (Union[int, str], optional): The version to use when interacting with the VirusTotal API.
                Defaults to 3.
            PROXIES (dict, optional): Proxies to use when making requests.
                E.g. `{"http": "http://10.10.1.10:3128", "https": "https://10.10.1.10:1080"}`
                Defaults to `None`.
            TIMEOUT (float, optional): The amount of time (in seconds) to wait for the HTTP request before timing out.
                Defaults to `None`.

        Raises:
            ValueError: When no `API_KEY` is provided or the `API_VERSION` is invalid.
        """
        if API_KEY is None:
            API_KEY = os.environ.get("VIRUSTOTAL_API_KEY", None)
            if API_KEY is None:
                raise ValueError(
                    "An API key is required to interact with the VirusTotal API.\nProvide one to the API_KEY parameter or by setting the environment variable 'VIRUSTOTAL_API_KEY'."
                )
        self.API_KEY = API_KEY
        self.PROXIES = PROXIES
        self.TIMEOUT = TIMEOUT
        # Declare appropriate variables depending on the API_VERSION provided
        if (API_VERSION == "v2") or (API_VERSION == 2):
            self.API_VERSION = API_VERSION
            self.BASEURL = "https://www.virustotal.com/vtapi/v2/"
            self.HEADERS = {
                "Accept-Encoding": "gzip, deflate",
                "User-Agent": f"gzip, virustotal-python 1.0.2",
            }
        elif (API_VERSION == "v3") or (API_VERSION == 3):
            self.API_VERSION = API_VERSION
            self.BASEURL = "https://www.virustotal.com/api/v3/"
            self.HEADERS = {
                "Accept-Encoding": "gzip, deflate",
                "User-Agent": f"gzip, virustotal-python 1.0.2",
                "x-apikey": f"{self.API_KEY}",
            }
        else:
            raise ValueError(
                f"The API version '{API_VERSION}' is not a valid VirusTotal API version.\nValid API versions are 'v2', 2, 'v3' and 3."
            )

    def __enter__(self):
        """Context Manager enter function."""
        return self

    def __exit__(self, type, value, traceback):
        """Context Manager exit function."""
        return

    def request(
        self,
        resource: str,
        params: dict = {},
        data: dict = None,
        json: dict = None,
        files: dict = None,
        method: str = "GET",
        large_file: bool = False,
    ) -> VirustotalResponse:
        """Make a request to the VirusTotal API.

        Args:
            resource (str): A valid VirusTotal API endpoint. E.g. `f'files/{id}'`.
            params (dict, optional): API endpoint query parameters. Defaults to `{}`.
            data (dict, optional): Data to send in the body of the request. Defaults to `None`.
            json (dict, optional): JSON payload to send with the request. Defaults to `None`.
            files (dict, optional): File(s) for multipart encoding upload. Defaults to `None`.
            method (str, optional): The HTTP request method to use. Defaults to `"GET"`.
            large_file (bool, optional): If a file is larger than 32MB, a custom generated upload URL is required.
                If this param is set to `True`, this URL can be set via the resource param. Defaults to `False`.

        Raises:
            NotImplementedError: When a unsupported HTTP method is provided.
            VirustotalError: When a failed VirusTotal API response occurs.

        Returns:
            VirustotalResponse: A `VirustotalResponse` class object.
        """
        endpoint = f"{self.BASEURL}{resource}"
        if large_file:
            endpoint = resource
        # VirusTotal API version 2 requires API_KEY to be in the request parameters
        if (self.API_VERSION == "v2") or (self.API_VERSION == 2):
            params["apikey"] = self.API_KEY
        if method == "GET":
            response = requests.get(
                endpoint,
                params=params,
                data=data,
                json=json,
                files=files,
                headers=self.HEADERS,
                proxies=self.PROXIES,
                timeout=self.TIMEOUT,
            )
        elif method == "POST":
            response = requests.post(
                endpoint,
                params=params,
                data=data,
                json=json,
                files=files,
                headers=self.HEADERS,
                proxies=self.PROXIES,
                timeout=self.TIMEOUT,
            )
        elif method == "PATCH":
            response = requests.patch(
                endpoint,
                params=params,
                data=data,
                json=json,
                files=files,
                headers=self.HEADERS,
                proxies=self.PROXIES,
                timeout=self.TIMEOUT,
            )
        elif method == "DELETE":
            response = requests.delete(
                endpoint,
                params=params,
                data=data,
                json=json,
                files=files,
                headers=self.HEADERS,
                proxies=self.PROXIES,
                timeout=self.TIMEOUT,
            )
        else:
            raise NotImplementedError(
                f"The request method '{method}' is not implemented."
            )
        if response.status_code != 200:
            raise VirustotalError(response)
        else:
            return VirustotalResponse(response)



"""A Python library to interact with the public VirusTotal v3 and v2 APIs.
"""
from virustotal_python.virustotal import Virustotal
from virustotal_python.virustotal import VirustotalResponse
from virustotal_python.virustotal import VirustotalError

name = "virustotal-python"

__all__ = ["Virustotal", "VirustotalResponse", "VirustotalError"]
__author__ = "dbrennand"
__version__ = "1.0.2"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['virustotal_python']

package_data = \
{'': ['*']}

install_requires = \
['requests[socks]>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'virustotal-python',
    'version': '1.0.2',
    'description': 'A Python library to interact with the public VirusTotal v3 and v2 APIs.',
    'long_description': '# virustotal-python 🐍\n![PyPI](https://img.shields.io/pypi/v/virustotal-python.svg?style=flat-square)\n![PyPI Stats](https://img.shields.io/pypi/dm/virustotal-python?color=blue&style=flat-square)\n[![CI](https://github.com/dbrennand/virustotal-python/actions/workflows/ci.yml/badge.svg)](https://github.com/dbrennand/virustotal-python/actions/workflows/ci.yml)\n[![Publish](https://github.com/dbrennand/virustotal-python/actions/workflows/publish.yml/badge.svg)](https://github.com/dbrennand/virustotal-python/actions/workflows/publish.yml)\n\nA Python library to interact with the public VirusTotal v3 and v2 APIs.\n\n> This library is intended to be used with the public VirusTotal APIs. However, it *could* be used to interact with premium API endpoints as well.\n>\n> It is highly recommended that you use the VirusTotal v3 API as it is the "default and encouraged way to programmatically interact with VirusTotal".\n\n## Installation 🛠\n\n```bash\n# PyPi\npip install virustotal-python\n# Manually\npip install .\n# Poetry\npoetry install --no-dev\n```\n\n## Get a VirusTotal API Key 🔑\n\n[Sign up](https://www.virustotal.com/gui/join-us) for a VirusTotal account. Then, view your VirusTotal API key.\n\n![VirusTotal view API key](images/APIKey.png)\n\n## Getting Started\n\n```python\nimport virustotal_python\n\nwith virustotal_python.Virustotal("<VirusTotal API Key>") as vtotal:\n    # Your code here...\n\n# Use the (old) VirusTotal version 2 API\nwith virustotal_python.Virustotal(\n    API_KEY="<VirusTotal API Key>", API_VERSION=2\n) as vtotal:\n    # Your code here...\n\n# You can also set proxies and timeouts for requests made by the library\n# NOTE: To use proxies, you must have the PySocks extra installed\nwith virustotal_python.Virustotal(\n    API_KEY="<VirusTotal API Key>",\n    PROXIES={"http": "http://10.10.1.10:3128", "https": "https://10.10.1.10:1080"},\n    TIMEOUT=5.0,\n) as vtotal:\n    # Your code here...\n\n# You can also omit the API_KEY parameter and provide your\n# API key via the environment variable VIRUSTOTAL_API_KEY\n# Bash: export VIRUSTOTAL_API_KEY="<VirusTotal API Key>"\n# PowerShell: $Env:VIRUSTOTAL_API_KEY = "<VirusTotal API Key>"\n# Then...\nwith virustotal_python.Virustotal() as vtotal:\n    # Your code here...\n```\n\n## Code Snippets\n\n> Further usage examples can be found in [examples](examples).\n\n### Send a file for analysis 🔎\n\n```python\nimport virustotal_python\nimport os.path\nfrom pprint import pprint\n\nFILE_PATH = "/path/to/file/to/scan.txt"\n\n# Create dictionary containing the file to send for multipart encoding upload\nfiles = {"file": (os.path.basename(FILE_PATH), open(os.path.abspath(FILE_PATH), "rb"))}\n\nwith virustotal_python.Virustotal("<VirusTotal API Key>") as vtotal:\n    resp = vtotal.request("files", files=files, method="POST")\n    pprint(resp.json())\n```\n\n### Get information about a file 📁\n\n```python\nimport virustotal_python\nfrom pprint import pprint\n\n# The ID (either SHA-256, SHA-1 or MD5 hash) identifying the file\nFILE_ID = "9f101483662fc071b7c10f81c64bb34491ca4a877191d464ff46fd94c7247115"\n\nwith virustotal_python.Virustotal("<VirusTotal API Key>") as vtotal:\n    resp = vtotal.request(f"files/{FILE_ID}")\n    pprint(resp.data)\n```\n\n### Send a URL 🔗 for analysis and get the report 📄\n\n```python\nimport virustotal_python\nfrom pprint import pprint\nfrom base64 import urlsafe_b64encode\n\nurl = "ihaveaproblem.info"\n\nwith virustotal_python.Virustotal("<VirusTotal API Key>") as vtotal:\n    try:\n        resp = vtotal.request("urls", data={"url": url}, method="POST")\n        # Safe encode URL in base64 format\n        # https://developers.virustotal.com/reference/url\n        url_id = urlsafe_b64encode(url.encode()).decode().strip("=")\n        report = vtotal.request(f"urls/{url_id}")\n        pprint(report.object_type)\n        pprint(report.data)\n    except virustotal_python.VirustotalError as err:\n        print(f"Failed to send URL: {url} for analysis and get the report: {err}")\n```\n\n### Get information about a domain:\n\n```python\nimport virustotal_python\nfrom pprint import pprint\n\ndomain = "virustotal.com"\n\nwith virustotal_python.Virustotal("<VirusTotal API Key>") as vtotal:\n    resp = vtotal.request(f"domains/{domain}")\n    pprint(resp.data)\n```\n\n## Development\n\n[Black](https://github.com/psf/black) is used for code formatting.\n\n### Unit Tests\n\nInstall the development dependencies using Poetry:\n\n```bash\npoetry install && poetry shell\n```\n\nTo run the unit tests, run `pytest` from the root of the project:\n\n```bash\npytest --cov=virustotal_python\n```\n\n### Publishing a new release\n\n```bash\n# Run from the master branch\nexport VERSION=x.x.x\ngit commit --allow-empty -m "Publish $VERSION"\ngit tag -a $VERSION -m "Version $VERSION"\ngit push --tags\n```\n\n## Authors & Contributors\n\n* [**dbrennand**](https://github.com/dbrennand) - *Author*\n\n* [**smk762**](https://github.com/smk762) - *Contributor*\n\n## Changelog\n\nSee the [CHANGELOG](CHANGELOG.md) for details.\n\n## License\nThis project is licensed under the MIT License - see the [LICENSE](LICENSE) for details.\n',
    'author': 'dbrennand',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/dbrennand/virustotal-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
